/**
    @file main.c
    @brief FT9xx UART to SPI Bridge
**/
/*
 * ============================================================================
 * History
 * =======
 * 2017-03-15 : Created
 *
 * (C) Copyright Bridgetek Pte Ltd
 * ============================================================================
 *
 * This source code ("the Software") is provided by Bridgetek Pte Ltd
 * ("Bridgetek") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of Bridgetek liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on Bridgetek's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, Bridgetek
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/
#include <stdint.h>
#include "ft900.h"

/* CONSTANTS ***********************************************************************/
#define SPI_SSx 3

/* GLOBAL VARIABLES ****************************************************************/

/* LOCAL VARIABLES *****************************************************************/
static uint8_t spi_channel = 0;

/* MACROS **************************************************************************/

/* LOCAL FUNCTIONS / INLINES *******************************************************/
void setup(void);
void loop(void);

void setup_uart(uint16_t divider);
void setup_spi(uint8_t channel, spi_clock_mode_t mode, uint16_t divider);

/* FUNCTIONS ***********************************************************************/

int main(void)
{
    setup();
    for(;;) loop();
    return 0;
}

void setup()
{
    /* Enable the UART Device... */
    setup_uart(UART_DIVIDER_19200_BAUD);

    /* Enable the SPI Device */
    setup_spi(SPI_SSx, spi_mode_0, 16);
}

void loop()
{
    static uint8_t bytes = 0;
    static uint8_t c;

    uart_read(UART0, &c);

    if (bytes == 0)
    {
        bytes = c;
        spi_open(SPIM, spi_channel);
    }
    else
    {
        spi_write(SPIM, c);
        spi_read(SPIM, c);

        bytes--;
        if (bytes == 0)
            spi_close(SPIM, spi_channel);

        uart_write(UART0, c);
    }

}

void setup_uart(uint16_t divider)
{
    sys_enable(sys_device_uart0);

    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */

    uart_open(UART0,                    /* Device */
              1,                        /* Prescaler = 1 */
              divider,
              uart_data_bits_8,         /* No. Data Bits */
              uart_parity_none,         /* Parity */
              uart_stop_bits_1);        /* No. Stop Bits */

    /* Print out a welcome message... */
        uart_puts(UART0,
            "\x1B[2J" /* ANSI/VT100 - Clear the Screen */
            "\x1B[H"  /* ANSI/VT100 - Move Cursor to Home */
            "(C) Copyright 2015, Bridgetek Pte Ltd. \r\n"
            "--------------------------------------------------------------------- \r\n"
            "Welcome to UART to SPI Bridge Application... \r\n"
            "\r\n"
            "Allows the SPI Master on the FT9xx chip to be controlled via commands \r\n"
        	"issued over a UART.\r\n"
            "--------------------------------------------------------------------- \r\n"
            );

}

void setup_spi(uint8_t channel, spi_clock_mode_t mode, uint16_t divider)
{
    /* Enable the SPI Device */
    sys_enable(sys_device_spi_master);

    gpio_function(27, pad_spim_sck);
    gpio_function(28, pad_spim_ss0);
    gpio_function(29, pad_spim_mosi);
    gpio_function(30, pad_spim_miso);
    gpio_function(31, pad_spim_io2);
    gpio_function(32, pad_spim_io3);
    gpio_function(33, pad_spim_ss1);
    gpio_function(34, pad_spim_ss2);
    gpio_function(35, pad_spim_ss3);

    spi_channel = channel;

    if (-1 == spi_init(SPIM, spi_dir_master, mode, divider))
        uart_puts(UART0, "spi_init ERROR\r\n");
}
